#!/usr/bin/python3

import os.path
import sys
import getopt
import requests
import re
import urllib.parse
import urllib3
import json

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

GOOROOM_INFO_FILE = "/etc/gooroom/info"

# read latest version of gooroom-info from repo server
SOURCES_LIST_FILE = "/etc/apt/sources.list"
SOURCES_LIST_PATH = "/etc/apt/sources.list.d/"
TMAX_SOURCES_LIST_NAME_PATTERN = "tmax"
TMAX_REPOSITORY_DIRECTORY = "/tmax"
LATEST_PATH = "release-notes/latest"
LATEST_KEY = "latestVersion"

# read latest OS version from GPMS server
GPMS_URL = "https://test-gpms.gooroom.kr/"
GPMS_INFO_PATH = "files/os_version.null"

def getCurrentVersion():
    currentVersion = ""
    
    if not os.path.isfile(GOOROOM_INFO_FILE):
        print("[ERROR] OS Version Data File Does Not Exist", file=sys.stderr)
        return

    with open(GOOROOM_INFO_FILE, 'r') as f:
        lines = f.readlines()
        for l in lines:
            if "RELEASE" in l:
                currentVersion = l.split("RELEASE=")[-1].strip()
                break
    
    if currentVersion == "":
        print("[ERROR] OS Version Data File is Damaged", file=sys.stderr)
        return
    
    return currentVersion

def getLatestFromReleaseNote():
    sourcesListFilesToCheck = []

    for sourcesList in os.listdir(SOURCES_LIST_PATH):
        if TMAX_SOURCES_LIST_NAME_PATTERN in sourcesList:
            sourcesListFilesToCheck.append(SOURCES_LIST_PATH + sourcesList)
    
    sourcesListFilesToCheck.append(SOURCES_LIST_FILE)

    for sourcesListFile in sourcesListFilesToCheck:
        if os.path.isfile(sourcesListFile):
            with open(sourcesListFile, 'r') as f:
                for repositoryLine in f.readlines():
                    repositoryLine = repositoryLine.strip()

                    if repositoryLine.startswith("#"):
                        continue
                    elif "deb" not in repositoryLine:
                        continue
                    elif TMAX_REPOSITORY_DIRECTORY not in repositoryLine:
                        continue
                        
                    urlData = urllib.parse.urlparse(repositoryLine.split()[1])
                    repo = urlData.geturl()
                    repo = repo.split(TMAX_REPOSITORY_DIRECTORY)[0]
                    if not repo.endswith("/"):
                        repo = repo + "/"
                    
                    latestPath = urllib.parse.urljoin(repo, LATEST_PATH)
                    latestData = requests.get(latestPath)
                    
                    if latestData.status_code != 200:
                        continue
                    
                    latestText = latestData.text.strip()
                    latestJson = json.loads(latestText)
                    
                    if LATEST_KEY not in latestJson.keys():
                        continue
                    
                    version = latestJson[LATEST_KEY]
                    return version
    
    raise Exception("[ERROR] Repository Does Not Have release-notes")

def getLatestFromGPMS():
    gpms = urllib.parse.urljoin(GPMS_URL, GPMS_INFO_PATH)

    try:
        original_text = requests.get(gpms, verify=False, timeout=3).text
    except requests.exceptions.RequestException:
        raise Exception("[ERROR] Please Check Internet Connection")
    
    if "Forbidden" in original_text:
        raise Exception("[ERROR] Latest OS Version Not Found")

    latest_version = original_text.strip()
    return latest_version

def getLatestVer():
    try:
        return getLatestFromReleaseNote()
    except Exception as erepo:
        try:
            return getLatestFromGPMS()
        except Exception as egpms:
            if "Internet" not in str(erepo):
                print(erepo, file=sys.stderr)
            else:
                print(egpms, file=sys.stderr)

def main(argv):
    FILE_NAME = argv[0]

    if len(argv) == 1:
        print("\n[USAGE]", "check-os-version", "{--current | --latest | --print}\n")
        return

    try:
        opts, args = getopt.getopt(argv[1:], "hcnp", ["help", "current", "latest", "print"])
    except getopt.GetoptError:
        print("\n[USAGE]", "check-os-version", "{--current | --latest | --print}\n")
        return

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            print("\n[USAGE]", "check-os-version", "{--current | --latest | --print}\n")
        
        elif opt in ("-c", "--current"):
            current_version = getCurrentVersion()
            if current_version:
                print(current_version)
        
        elif opt in ("-n", "--latest"):
            latest_version = getLatestVer()
            if latest_version:
                print(latest_version)
    
        elif opt in ("-p", "--print"):
            print()
            print("OS Version Information")
            current_version = getCurrentVersion()
            print("- Current Version: {}".format(current_version))
            latest_version = getLatestVer()
            print("- Latest Version : {}".format(latest_version))
            print()

if __name__ == "__main__":
    main(sys.argv)
